{***************************************************************************
 *                                                                         *
 *           Unit de gestion des types pointeurs de POWER-GRAPH           *
 *                                                                         *
 ***************************************************************************}

{$A+,B-,D+,E-,F+,G+,I+,L+,N-,O-,R-,S+,V-,X-}
{$M 16384,0,655360}

UNIT PG_PTRS;

INTERFACE

 USES
     Graph;

 CONST MaxDeg=4;


 TYPE
     Palette=RECORD
                   Values:PaletteType;
                   Names:ARRAY[1..MaxDeg] OF STRING[10];
             END;
     Vec3=ARRAY[1..3] OF REAL;

     RealPtr=^REAL;
     EquPtr=^Equ;
     ProjectPtr=^Project;
     XPtr=^XPol;
     XYPtr=^XYPol;
     XYZPtr=^XYZPol;
     VecPtr=^Vec;
     ProjListPtr=^ProjList;
     CProjectPtr=^CProject;
     CEquPtr=^CEqu;
     StructPtr=^Struct;

     Equ=RECORD P   :XYZPtr;
                Size:WORD;
                Name:STRING[20];
                XPos,YPos:BYTE;
                Next:EquPtr;
                END;

     Project=RECORD P:XYZPtr;
                    Size:WORD;
                    Name:STRING[20];
                    XPos,YPos:BYTE;
                    Coul:BYTE;
                    Cond:EquPtr;
                    Next:ProjectPtr;
                    END;

     XPol=RECORD x:WORD;
                 c:REAL;
                 Next:XPtr;
                 END;

     XYPol=RECORD x:WORD;
                  y:WORD;
                  c:REAL;
                  Next:XYPtr;
                  END;

     XYZPol=RECORD x:WORD;
                   y:WORD;
                   z:WORD;
                   c:REAL;
                   Next:XYZPtr;
                   END;

     Vec=RECORD Val:REAL;
                Next:VecPtr;
                END;

     ProjList=RECORD
                    Proj:ProjectPtr;
                    Name:STRING[20];
                    YPos:BYTE;
                    Pal :Palette;
                    Next:ProjListPtr;
              END;



    CEqu=RECORD P   :XYZPtr;
                QX  :XPtr;
                QXY :XYPtr;
                QXYZ:XYZPtr;
                Q,QY:XPtr;
                Next:CEquPtr;
                END;

    CProject=RECORD P   :XYZPtr;
                    QX  :XPtr;
                    QXY :XYPtr;
                    QXYZ:XYZPtr;
                    Q,QY:XPtr;
                    Nx,Ny,Nz:XYZPtr;
                    Coul:BYTE;
                    Cond:CEquPtr;
                    Next:CProjectPtr;
                    END;

    Struct=RECORD Vals:VecPtr;
                  Eq:CProjectPtr;
                  Next:StructPtr;
                  END;

 PROCEDURE NewX(VAR P:XPtr);
 PROCEDURE NewXY(VAR P:XYPtr);
 PROCEDURE NewXYZ(VAR P:XYZPtr);
 PROCEDURE NewVec(VAR V:VecPtr);
 PROCEDURE NewProject(VAR N:ProjectPtr);
 PROCEDURE NewCProject(VAR N:CProjectPtr);
 PROCEDURE NewEqu(VAR E:EquPtr);
 PROCEDURE NewCEqu(VAR E:CEquPtr);
 PROCEDURE DelXPol(VAR P:XPtr);
 PROCEDURE DelXY(VAR P:XYPtr);
 PROCEDURE DelXYZ(VAR P:XYZPtr);
 PROCEDURE DelVec(VAR V:VecPtr);
 PROCEDURE DelEqu(VAR n:EquPtr);
 PROCEDURE DelCEqu(VAR n:CEquPtr);
 PROCEDURE DelProject(VAR n:ProjectPtr);
 PROCEDURE DelCProject(VAR n:CProjectPtr);
 PROCEDURE DelProjList(l:ProjListPtr);
 FUNCTION NthProjList(l:ProjListPtr;n:BYTE):ProjListPtr;
 FUNCTION NthProject(l:ProjectPtr;n:BYTE):ProjectPtr;
 FUNCTION NthEqu(l:EquPtr;n:BYTE):EquPtr;
 FUNCTION LastProjList(l:ProjListPtr):ProjListPtr;


IMPLEMENTATION

 PROCEDURE NewX(VAR P:XPtr);

  BEGIN New(P);
        P^.Next:=NIL;
        P^.x:=0;P^.c:=0;
  END; {NewX}



 PROCEDURE NewXY(VAR P:XYPtr);

  BEGIN New(P);
        P^.Next:=NIL;
        P^.x:=0;P^.y:=0;
        P^.c:=0;
  END; {NewXY}


 PROCEDURE NewXYZ(VAR P:XYZPtr);

  BEGIN New(P);
        P^.Next:=NIL;
        P^.x:=0;P^.y:=0;P^.z:=0;
        P^.c:=0;
  END; {NewXYZ}


 PROCEDURE NewVec(VAR V:VecPtr);

  BEGIN New(V);
        V^.Next:=NIL;
  END; {NewVec}


 PROCEDURE NewProject(VAR N:ProjectPtr);

  BEGIN New(N);
        WITH N^ DO BEGIN
             P:=NIL;
             Next:=NIL;
             Cond:=NIL;Coul:=0;
        END;
  END; {NewProject}


 PROCEDURE NewCProject(VAR N:CProjectPtr);

  BEGIN New(N);
        WITH N^ DO BEGIN
             P:=NIL;QX:=NIL;QXY:=NIL;QXYZ:=NIL;
             Q:=NIL;QY:=NIL;
             Next:=NIL;Nx:=NIL;Ny:=NIL;Nz:=NIL;
             Cond:=NIL;Coul:=0;
        END;
  END; {NewCProject}


 PROCEDURE NewEqu(VAR E:EquPtr);

  BEGIN New(E);
        WITH E^ DO BEGIN
             P:=NIL;
             Next:=NIL;
        END;
  END; {NewEqu}


 PROCEDURE NewCEqu(VAR E:CEquPtr);

  BEGIN New(E);
        WITH E^ DO BEGIN
             P:=NIL;QX:=NIL;QXY:=NIL;QXYZ:=NIL;
             Q:=NIL;QY:=NIL;
             Next:=NIL;
        END;
  END; {NewCEqu}


 PROCEDURE DelXPol(VAR P:XPtr);

  VAR PN:XPtr;

  BEGIN
       WHILE p<>NIL DO BEGIN
              PN:=P^.Next;
              DISPOSE(P);
              P:=PN;
       END;
  END; {DelXPol}


 PROCEDURE DelXY(VAR P:XYPtr);

  VAR PN:XYPtr;

  BEGIN
       WHILE p<>NIL DO BEGIN
              PN:=P^.Next;
              DISPOSE(P);
              P:=PN;
       END;
  END; {DelXYPol}


 PROCEDURE DelXYZ(VAR P:XYZPtr);

  VAR PN:XYZPtr;

  BEGIN
       WHILE p<>NIL DO BEGIN
              PN:=P^.Next;
              DISPOSE(P);
              P:=PN;
       END;
  END; {DelXYZ}


 PROCEDURE DelVec(VAR V:VecPtr);

  VAR VN:VecPtr;

  BEGIN
       WHILE v<>NIL DO BEGIN
              VN:=V^.Next;
              DISPOSE(V);
              V:=VN;
       END;
  END; {DelVec}


 PROCEDURE DelEqu(VAR n:EquPtr);

  VAR NN:EquPtr;

  BEGIN
       WHILE n<>NIL DO BEGIN
             NN:=n^.Next;
             DelXYZ(n^.p);
             DISPOSE(n);
             n:=nn;
       END;
  END; {DelEqu}


 PROCEDURE DelCEqu(VAR n:CEquPtr);

  VAR NN:CEquPtr;

  BEGIN
       WHILE n<>NIL DO BEGIN
             NN:=n^.Next;
             DelXPol(n^.qx);
             DelXY(n^.qxy);
             DelXYZ(n^.qxyz);
             DISPOSE(n);
             n:=nn;
       END;
  END; {DelCEqu}


 PROCEDURE DelProject(VAR n:ProjectPtr);

  VAR NN:ProjectPtr;

  BEGIN
       WHILE n<>NIL DO BEGIN
             NN:=n^.Next;
             DelXYZ(n^.p);
             DelEqu(n^.Cond);
             DISPOSE(n);
             n:=nn;
       END;
  END; {DelProject}


 PROCEDURE DelCProject(VAR n:CProjectPtr);

  VAR NN:CProjectPtr;

  BEGIN
       WHILE n<>NIL DO BEGIN
             NN:=n^.Next;
             DelXPol(n^.qx);
             DelXY(n^.qxy);
             DelXYZ(n^.qxyz);
             DelXYZ(n^.nx);
             DelXYZ(n^.ny);
             DelXYZ(n^.nz);
             DelCEqu(n^.Cond);
             DISPOSE(n);
             n:=nn;
       END;
  END; {DelCProject}


 PROCEDURE DelProjList(l:ProjListPtr);

  BEGIN
       IF l<>NIL THEN
          BEGIN
               DelProject(l^.Proj);
               DelProjList(l^.Next);
          END;
  END;


 FUNCTION NthProjList(l:ProjListPtr;n:BYTE):ProjListPtr;

  BEGIN
       IF l=NIL THEN NthProjList:=NIL
                ELSE IF n=1 THEN NthProjList:=l
                            ELSE NthProjList:=NthProjList(l^.Next,n-1);
  END; {NthProjList}


 FUNCTION LastProjList(l:ProjListPtr):ProjListPtr;

  BEGIN
       IF l=NIL
          THEN LastProjList:=NIL
          ELSE
              BEGIN
                   WHILE l^.Next<>NIL
                         DO l:=l^.Next;
                   LastProjList:=l;
              END;
  END; {LastProjList}


 FUNCTION NthProject(l:ProjectPtr;n:BYTE):ProjectPtr;

  BEGIN
       IF l=NIL THEN NthProject:=NIL
                ELSE IF n=1 THEN NthProject:=l
                            ELSE NthProject:=NthProject(l^.Next,n-1);
  END; {NthProject}


 FUNCTION NthEqu(l:EquPtr;n:BYTE):EquPtr;

  BEGIN
       IF l=NIL THEN NthEqu:=NIL
                ELSE IF n=1 THEN NthEqu:=l
                            ELSE NthEqu:=NthEqu(l^.Next,n-1);
  END; {NthEqu}

END.
